package cokr.xit.fims.excl.dao;

import java.util.List;

import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import cokr.xit.fims.crdn.Crdn;
import cokr.xit.fims.excl.LevyExcl;
import cokr.xit.fims.excl.LevyExclQuery;
import cokr.xit.foundation.component.AbstractMapper;
import cokr.xit.foundation.data.DataObject;

/** 부과제외 대장 정보 DAO
 *
 * <p>상세 설명: 부과제외 대장 테이블에 접근하여 데이터를 조회하고 저장한다.
 *
 * <pre>
 * ============ 변경 이력 ============
 * 2023-06-23	JoJH 최초 작성
 * ================================
 * </pre>
 */
@Mapper("levyExclMapper")
public interface LevyExclMapper extends AbstractMapper {

	/**지정한 조건에 따라 부과제외 대장 목록을 조회하여 반환한다.
	 * @param req 부과제외 대장 조회 조건
	 * @return 부과제외 대장 목록
	 */
	List<DataObject> selectLevyExclList(LevyExclQuery req);

	/**지정한 조건에 따라 부과제외 대장 객체들을 조회하여 반환한다.
	 * @param req 부과제외 대장 조회 조건
	 * @return 부과제외 대장 객체 정보
	 */
	List<DataObject> selectLevyExcls(LevyExclQuery req);

	/**지정한 조건에 따라 부과제외 대장 정보를 조회하여 반환한다.
	 * @param req 부과제외 대장 조회 조건
	 * @return 부과제외 대장 정보
	 */
	default DataObject selectLevyExclInfo(LevyExclQuery req) {
		// 삭제 여부 확인
		if (req.getDelYn() == null) {
			req.setDelYn("N");
		}
		// 기본 정렬
		if (req.getOrderBy() == null) {
			req.setOrderBy("LE.LEVY_EXCL_ID DESC");
		}

		List<DataObject> levyExcls = selectLevyExcls(req);

		return !levyExcls.isEmpty() ? levyExcls.get(0) : null;
	}

	/**부과제외 대장 정보를 등록한다.
	 * @param params 파라미터
	 * <ul><li>"levyExcl" - 부과제외 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int insertLevyExcl(LevyExcl levyExcl);

	/**부과제외 대장 정보를 수정한다.
	 * @param params 파라미터
	 * <ul><li>"levyExcl" - 부과제외 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int updateLevyExcl(LevyExcl levyExcl);

	/**부과제외 대장 정보를 삭제한다.
	 * @param params 파라미터
	 * <ul><li>"levyExcl" - 부과제외 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int deleteLevyExcl(LevyExcl levyExcl);

	/**단속(TB_CRDN) 대장의 주차가능결과코드를 수정한다.
	 * @param crdn 단속 대장
	 * @return 저장된 정보수
	 */
	int updatePrkPsbltyRsltCd(Crdn crdn);

}
